@@ -31,7 +31,6 @@ import (
 	"time"
 
 	"go.temporal.io/api/common/v1"
-	commonpb "go.temporal.io/api/common/v1"
 	enumspb "go.temporal.io/api/enums/v1"
 	schedpb "go.temporal.io/api/schedule/v1"
 	"go.temporal.io/api/taskqueue/v1"
@@ -43,6 +42,7 @@ import (
 	"github.com/pborman/uuid"
 	"github.com/temporalio/tctl-kit/pkg/color"
 	"github.com/temporalio/tctl-kit/pkg/output"
+	"github.com/temporalio/tctl-kit/pkg/pager"
 	"github.com/urfave/cli/v2"
 	"go.temporal.io/api/workflowservice/v1"
 )
@@ -54,13 +54,13 @@ func scheduleBaseArgs(c *cli.Context) (
 	err error,
 ) {
 	frontendClient = cFactory.FrontendClient(c)
-	namespace, err = getRequiredGlobalOption(c, FlagNamespace)
+	namespace, err = requiredFlag(c, FlagNamespace)
 	if err != nil {
 		return nil, "", "", err
 	}
-	scheduleID = c.String(FlagScheduleID)
-	if scheduleID == "" {
-		return nil, "", "", errors.New("empty schedule id")
+	scheduleID, err = requiredFlag(c, FlagScheduleID)
+	if err != nil {
+		return nil, "", "", err
 	}
 	return frontendClient, namespace, scheduleID, nil
 }
@@ -102,6 +102,7 @@ func buildScheduleSpec(c *cli.Context) (*schedpb.ScheduleSpec, error) {
 		}
 		out.Calendar = append(out.Calendar, cal)
 	}
+	out.CronString = c.StringSlice(FlagCronSchedule)
 	for _, s := range c.StringSlice(FlagInterval) {
 		cal, err := buildIntervalSpec(s)
 		if err != nil {
@@ -133,7 +134,7 @@ func buildScheduleSpec(c *cli.Context) (*schedpb.ScheduleSpec, error) {
 	if c.IsSet(FlagTimeZone) {
 		tzName := c.String(FlagTimeZone)
 		if _, err := time.LoadLocation(tzName); err != nil {
-			return nil, fmt.Errorf("Unknown time zone name %q", tzName)
+			return nil, fmt.Errorf("unknown time zone name %q", tzName)
 		}
 		out.TimezoneName = tzName
 	}
@@ -168,8 +169,8 @@ func buildScheduleAction(c *cli.Context) (*schedpb.ScheduleAction, error) {
 
 func buildScheduleState(c *cli.Context) (*schedpb.ScheduleState, error) {
 	var out schedpb.ScheduleState
-	out.Notes = c.String(FlagInitialNotes)
-	out.Paused = c.Bool(FlagInitialPaused)
+	out.Notes = c.String(FlagNotes)
+	out.Paused = c.Bool(FlagPause)
 	if c.IsSet(FlagRemainingActions) {
 		out.LimitedActions = true
 		out.RemainingActions = int64(c.Int(FlagRemainingActions))
@@ -221,7 +222,7 @@ func buildSchedule(c *cli.Context) (*schedpb.Schedule, error) {
 	return sched, nil
 }
 
-func getMemoAndSearchAttributesForSchedule(c *cli.Context) (*commonpb.Memo, *commonpb.SearchAttributes, error) {
+func getMemoAndSearchAttributesForSchedule(c *cli.Context) (*common.Memo, *common.SearchAttributes, error) {
 	if memoMap, err := unmarshalMemoFromCLI(c); err != nil {
 		return nil, nil, err
 	} else if memo, err := encodeMemo(memoMap); err != nil {
@@ -264,7 +265,7 @@ func CreateSchedule(c *cli.Context) error {
 
 	_, err = frontendClient.CreateSchedule(ctx, req)
 	if err != nil {
-		return fmt.Errorf("unable to create schedule: %s", err)
+		return fmt.Errorf("unable to create schedule: %w", err)
 	}
 
 	fmt.Println(color.Green(c, "Schedule created"))
@@ -294,7 +295,7 @@ func UpdateSchedule(c *cli.Context) error {
 
 	_, err = frontendClient.UpdateSchedule(ctx, req)
 	if err != nil {
-		return fmt.Errorf("failed to update schedule.\n%s", err)
+		return fmt.Errorf("unable to update schedule: %w", err)
 	}
 
 	fmt.Println(color.Green(c, "Schedule updated"))
@@ -331,7 +332,7 @@ func ToggleSchedule(c *cli.Context) error {
 	}
 	_, err = frontendClient.PatchSchedule(ctx, req)
 	if err != nil {
-		return fmt.Errorf("failed to toggle schedule.\n%s", err)
+		return fmt.Errorf("unable to toggle schedule: %w", err)
 	}
 
 	fmt.Println(color.Green(c, "Schedule updated"))
@@ -364,7 +365,7 @@ func TriggerSchedule(c *cli.Context) error {
 	}
 	_, err = frontendClient.PatchSchedule(ctx, req)
 	if err != nil {
-		return fmt.Errorf("failed to trigger schedule.\n%s", err)
+		return fmt.Errorf("unable to trigger schedule: %w", err)
 	}
 
 	fmt.Println(color.Green(c, "Trigger request sent"))
@@ -398,7 +399,7 @@ func BackfillSchedule(c *cli.Context) error {
 		ScheduleId: scheduleID,
 		Patch: &schedpb.SchedulePatch{
 			BackfillRequest: []*schedpb.BackfillRequest{
-				&schedpb.BackfillRequest{
+				{
 					StartTime:     timestamp.TimePtr(startTime),
 					EndTime:       timestamp.TimePtr(endTime),
 					OverlapPolicy: overlap,
@@ -410,7 +411,7 @@ func BackfillSchedule(c *cli.Context) error {
 	}
 	_, err = frontendClient.PatchSchedule(ctx, req)
 	if err != nil {
-		return fmt.Errorf("failed to backfill schedule.\n%s", err)
+		return fmt.Errorf("unable to backfill schedule: %w", err)
 	}
 
 	fmt.Println(color.Green(c, "Backfill request sent"))
@@ -431,7 +432,7 @@ func DescribeSchedule(c *cli.Context) error {
 	}
 	resp, err := frontendClient.DescribeSchedule(ctx, req)
 	if err != nil {
-		return fmt.Errorf("failed to describe schedule.\n%s", err)
+		return fmt.Errorf("unable to describe schedule: %w", err)
 	}
 
 	if c.Bool(FlagPrintRaw) {
@@ -465,7 +466,7 @@ func DescribeSchedule(c *cli.Context) error {
 		// more convenient copies of values from Info
 		NextRunTime       *time.Time
 		LastRunTime       *time.Time
-		LastRunExecution  *commonpb.WorkflowExecution
+		LastRunExecution  *common.WorkflowExecution
 		LastRunActualTime *time.Time
 
 		Memo             map[string]string // json only
@@ -475,6 +476,7 @@ func DescribeSchedule(c *cli.Context) error {
 	s, i := resp.Schedule, resp.Info
 	item.ScheduleId = scheduleID
 	item.Specification = s.Spec
+	uncanonicalizeSpec(item.Specification)
 	if sw := s.Action.GetStartWorkflow(); sw != nil {
 		item.StartWorkflow = sw
 		item.WorkflowType = sw.WorkflowType.GetName()
@@ -535,8 +537,7 @@ func DescribeSchedule(c *cli.Context) error {
 			"Info.InvalidScheduleError",
 		},
 	}
-	output.PrintItems(c, []interface{}{item}, opts)
-	return nil
+	return output.PrintItems(c, []interface{}{item}, opts)
 }
 
 func DeleteSchedule(c *cli.Context) error {
@@ -554,7 +555,7 @@ func DeleteSchedule(c *cli.Context) error {
 	}
 	_, err = frontendClient.DeleteSchedule(ctx, req)
 	if err != nil {
-		return fmt.Errorf("failed to delete schedule.\n%s", err)
+		return fmt.Errorf("unable to delete schedule: %w", err)
 	}
 
 	fmt.Println(color.Green(c, "Schedule deleted"))
@@ -563,7 +564,7 @@ func DeleteSchedule(c *cli.Context) error {
 
 func ListSchedules(c *cli.Context) error {
 	frontendClient := cFactory.FrontendClient(c)
-	namespace, err := getRequiredGlobalOption(c, FlagNamespace)
+	namespace, err := requiredFlag(c, FlagNamespace)
 	if err != nil {
 		return err
 	}
@@ -579,7 +580,7 @@ func ListSchedules(c *cli.Context) error {
 		}
 		resp, err := frontendClient.ListSchedules(ctx, req)
 		if err != nil {
-			return nil, nil, fmt.Errorf("failed to list schedules.\n%s", err)
+			return nil, nil, fmt.Errorf("unable to list schedules: %w", err)
 		}
 		items := make([]interface{}, len(resp.Schedules))
 		for i, sch := range resp.Schedules {
@@ -596,7 +597,7 @@ func ListSchedules(c *cli.Context) error {
 				Info struct {
 					NextRunTime       *time.Time
 					LastRunTime       *time.Time
-					LastRunExecution  *commonpb.WorkflowExecution
+					LastRunExecution  *common.WorkflowExecution
 					LastRunActualTime *time.Time
 				}
 			}
@@ -618,6 +619,7 @@ func ListSchedules(c *cli.Context) error {
 				item.Info.LastRunExecution = ra.StartWorkflowResult
 			}
 			item.Specification = info.GetSpec()
+			uncanonicalizeSpec(item.Specification)
 			items[i] = item
 		}
 		return items, resp.NextPageToken, nil
@@ -627,11 +629,43 @@ func ListSchedules(c *cli.Context) error {
 	opts := &output.PrintOptions{
 		Fields:     []string{"ScheduleId", "StartWorkflow.WorkflowType", "State.Paused", "State.Notes", "Info.NextRunTime", "Info.LastRunTime"},
 		FieldsLong: []string{"Info.LastRunActualTime", "Info.LastRunExecution", "Specification"},
+		Pager:      pager.Less,
 	}
 	if missingExtendedInfo {
 		fmt.Println(color.Yellow(c, "Note: Extended schedule information is not available without Elasticsearch"))
 		opts.Fields = []string{"ScheduleId"}
 		opts.FieldsLong = nil
 	}
-	return output.Pager(c, iter, opts)
+	return output.PrintIterator(c, iter, opts)
+}
+
+func uncanonicalizeSpec(spec *schedpb.ScheduleSpec) {
+	processField := func(ranges []*schedpb.Range) string {
+		var out []string
+		for _, r := range ranges {
+			s := fmt.Sprintf("%d", r.Start)
+			if r.End > r.Start {
+				s += fmt.Sprintf("-%d", r.End)
+			}
+			if r.Step > 1 {
+				s += fmt.Sprintf("/%d", r.Step)
+			}
+			out = append(out, s)
+		}
+		return strings.Join(out, ",")
+	}
+	// Turn StructuredCalenderSpec into CalendarSpec for ease of reading
+	for _, scs := range spec.StructuredCalendar {
+		spec.Calendar = append(spec.Calendar, &schedpb.CalendarSpec{
+			Second:     processField(scs.Second),
+			Minute:     processField(scs.Minute),
+			Hour:       processField(scs.Hour),
+			DayOfMonth: processField(scs.DayOfMonth),
+			Month:      processField(scs.Month),
+			Year:       processField(scs.Year),
+			DayOfWeek:  processField(scs.DayOfWeek),
+			Comment:    scs.Comment,
+		})
+	}
+	spec.StructuredCalendar = nil
 }
