@@ -28,6 +28,7 @@ import (
 	"bufio"
 	"context"
 	"encoding/json"
+	"errors"
 	"fmt"
 	"os"
 	"os/user"
@@ -38,19 +39,18 @@ import (
 
 	"github.com/fatih/color"
 	"github.com/gogo/protobuf/proto"
+	tColor "github.com/temporalio/tctl-kit/pkg/color"
+	"github.com/temporalio/tctl/v2/cli/dataconverter"
+	"github.com/temporalio/tctl/v2/cli/headers"
+	"github.com/temporalio/tctl/v2/cli/stringify"
 	"github.com/urfave/cli/v2"
 	commonpb "go.temporal.io/api/common/v1"
 	enumspb "go.temporal.io/api/enums/v1"
 	historypb "go.temporal.io/api/history/v1"
 	sdkclient "go.temporal.io/sdk/client"
 	"go.temporal.io/sdk/converter"
-
 	"go.temporal.io/server/common/codec"
 	"go.temporal.io/server/common/payloads"
-
-	"github.com/temporalio/tctl/cli/dataconverter"
-	"github.com/temporalio/tctl/cli/headers"
-	"github.com/temporalio/tctl/cli/stringify"
 )
 
 // HistoryEventToString convert HistoryEvent to string
@@ -328,7 +328,7 @@ func getCurrentUserFromEnv() string {
 			return os.Getenv(n)
 		}
 	}
-	return "unkown"
+	return "unknown"
 }
 
 func prettyPrintJSONObject(o interface{}) {
@@ -342,7 +342,7 @@ func prettyPrintJSONObject(o interface{}) {
 	}
 
 	if err != nil {
-		fmt.Printf("Error when try to print pretty: %v\n", err)
+		fmt.Printf("Error when try to print pretty: %v", err)
 		fmt.Println(o)
 	}
 	_, _ = os.Stdout.Write(b)
@@ -358,40 +358,21 @@ func mapKeysToArray(m map[string]interface{}) []string {
 }
 
 func getSDKClient(c *cli.Context) (sdkclient.Client, error) {
-	namespace, err := getRequiredGlobalOption(c, FlagNamespace)
+	namespace, err := requiredFlag(c, FlagNamespace)
 	if err != nil {
 		return nil, err
 	}
 	return cFactory.SDKClient(c, namespace), nil
 }
 
-func getRequiredGlobalOption(c *cli.Context, optionName string) (string, error) {
-	value := readFlagOrConfig(c, optionName)
+func requiredFlag(c *cli.Context, optionName string) (string, error) {
+	value := c.String(optionName)
 	if len(value) == 0 {
-		return "", fmt.Errorf("global option is required: %s", optionName)
+		return "", fmt.Errorf("option %v is required: ", tColor.Yellow(c, "--%v", optionName))
 	}
 	return value, nil
 }
 
-func readFlagOrConfig(c *cli.Context, key string) string {
-	if c.IsSet(key) {
-		return c.String(key)
-	}
-
-	var cVal string
-	if isRootKey(key) {
-		cVal, _ = tctlConfig.Get(c, key)
-	} else if isEnvKey(key) {
-		cVal, _ = tctlConfig.GetByEnvironment(c, key)
-	}
-
-	if cVal != "" {
-		return cVal
-	}
-
-	return c.String(key)
-}
-
 func formatTime(t time.Time, onlyTime bool) string {
 	var result string
 	if onlyTime {
@@ -583,7 +564,7 @@ func unmarshalInputsFromCLI(c *cli.Context) ([]interface{}, error) {
 		} else {
 			var j interface{}
 			if err := json.Unmarshal(jsonRaw, &j); err != nil {
-				return nil, fmt.Errorf("input is not valid JSON: %s", err)
+				return nil, fmt.Errorf("input is not valid JSON: %w", err)
 			}
 			result = append(result, j)
 		}
@@ -601,7 +582,7 @@ func processJSONInput(c *cli.Context) (*commonpb.Payloads, error) {
 	}
 	p, err := payloads.Encode(jsons...)
 	if err != nil {
-		return nil, fmt.Errorf("unable to encode input: %s", err)
+		return nil, fmt.Errorf("unable to encode input: %w", err)
 	}
 
 	return p, nil
@@ -637,7 +618,7 @@ func readJSONInputs(c *cli.Context) ([][]byte, error) {
 		// #nosec
 		data, err := os.ReadFile(inputFile)
 		if err != nil {
-			return nil, fmt.Errorf("unable to read input file: %s", err)
+			return nil, fmt.Errorf("unable to read input file: %w", err)
 		}
 		return [][]byte{data}, nil
 	}
@@ -715,7 +696,7 @@ func prompt(msg string, autoConfirm bool, expectedInputs ...string) bool {
 		return true
 	}
 
-	textLower := strings.ToLower(strings.TrimRight(text, "\n"))
+	textLower := strings.ToLower(strings.TrimSpace(text))
 	for _, expectedInput := range expectedInputs {
 		if expectedInput == textLower {
 			return true
@@ -761,3 +742,57 @@ func ensureNonNil[T any, P ~*T](ptr *P) {
 		*ptr = new(T)
 	}
 }
+
+func listWorkflowExecutionStatusNames() string {
+	var names []string
+	for _, name := range enumspb.WorkflowExecutionStatus_name {
+		names = append(names, strings.ToLower(name))
+	}
+	return strings.Join(names, ", ")
+}
+
+// findWorkflowStatusValue finds a WorkflowExecutionStatus by its name. This search is case-insensitive.
+func findWorkflowStatusValue(name string) (enumspb.WorkflowExecutionStatus, bool) {
+	lowerName := strings.ToLower(name)
+	for key, value := range enumspb.WorkflowExecutionStatus_value {
+		if lowerName == strings.ToLower(key) {
+			return enumspb.WorkflowExecutionStatus(value), true
+		}
+	}
+
+	return 0, false
+}
+
+func parseFoldStatusList(flagValue string) ([]enumspb.WorkflowExecutionStatus, error) {
+	var statusList []enumspb.WorkflowExecutionStatus
+	for _, value := range strings.Split(flagValue, ",") {
+		if status, ok := findWorkflowStatusValue(value); ok {
+			statusList = append(statusList, status)
+		} else {
+			return nil,
+				fmt.Errorf("invalid status \"%s\" for fold flag. Valid values: %v", value, listWorkflowExecutionStatusNames())
+		}
+	}
+	return statusList, nil
+}
+
+// SplitKeyValuePairs parses key=value pairs
+func SplitKeyValuePairs(kvs []string) (map[string]string, error) {
+	pairs := make(map[string]string, len(kvs))
+	for _, v := range kvs {
+		parts := strings.SplitN(v, "=", 2)
+		if len(parts) != 2 {
+			return nil, fmt.Errorf("unable to split key=value pair: %v", v)
+		}
+		key := strings.TrimSpace(parts[0])
+		value := strings.TrimSpace(parts[1])
+
+		if key == "" {
+			return nil, errors.New("empty key is not allowed")
+		}
+
+		pairs[key] = value
+	}
+
+	return pairs, nil
+}
