@@ -40,12 +40,12 @@ import (
 	"github.com/pborman/uuid"
 	"github.com/temporalio/tctl-kit/pkg/color"
 	"github.com/temporalio/tctl-kit/pkg/output"
+	"github.com/temporalio/tctl-kit/pkg/pager"
 	"github.com/urfave/cli/v2"
 	commonpb "go.temporal.io/api/common/v1"
 	enumspb "go.temporal.io/api/enums/v1"
 	failurepb "go.temporal.io/api/failure/v1"
 	historypb "go.temporal.io/api/history/v1"
-	"go.temporal.io/api/operatorservice/v1"
 	querypb "go.temporal.io/api/query/v1"
 	"go.temporal.io/api/serviceerror"
 	workflowpb "go.temporal.io/api/workflow/v1"
@@ -54,14 +54,12 @@ import (
 	clispb "go.temporal.io/server/api/cli/v1"
 	"go.temporal.io/server/common"
 	"go.temporal.io/server/common/backoff"
-	"go.temporal.io/server/common/clock"
 	"go.temporal.io/server/common/collection"
 	"go.temporal.io/server/common/convert"
 	"go.temporal.io/server/common/primitives/timestamp"
 	"go.temporal.io/server/common/searchattribute"
-	"go.temporal.io/server/service/history/workflow"
 
-	"github.com/temporalio/tctl/cli/stringify"
+	"github.com/temporalio/tctl/v2/cli/stringify"
 )
 
 func startWorkflowBaseArgs(c *cli.Context) (
@@ -71,7 +69,7 @@ func startWorkflowBaseArgs(c *cli.Context) (
 	wid string,
 ) {
 	taskQueue = c.String(FlagTaskQueue)
-	workflowType = c.String(FlagWorkflowType)
+	workflowType = c.String(FlagType)
 	et = c.Int(FlagWorkflowExecutionTimeout)
 	rt = c.Int(FlagWorkflowRunTimeout)
 	dt = c.Int(FlagWorkflowTaskTimeout)
@@ -89,7 +87,7 @@ func StartWorkflow(c *cli.Context, printProgress bool) error {
 		return err
 	}
 
-	namespace, err := getRequiredGlobalOption(c, FlagNamespace)
+	namespace, err := requiredFlag(c, FlagNamespace)
 	if err != nil {
 		return err
 	}
@@ -100,7 +98,7 @@ func StartWorkflow(c *cli.Context, printProgress bool) error {
 	if c.IsSet(FlagWorkflowIDReusePolicy) {
 		reusePolicyInt, err := stringToEnum(c.String(FlagWorkflowIDReusePolicy), enumspb.WorkflowIdReusePolicy_value)
 		if err != nil {
-			return fmt.Errorf("unable to parse workflow ID reuse policy: %s", err)
+			return fmt.Errorf("unable to parse workflow ID reuse policy: %w", err)
 		}
 		reusePolicy = enumspb.WorkflowIdReusePolicy(reusePolicyInt)
 	}
@@ -136,7 +134,7 @@ func StartWorkflow(c *cli.Context, printProgress bool) error {
 	resp, err := sdkClient.ExecuteWorkflow(tcCtx, wo, workflowType, inputs...)
 
 	if err != nil {
-		return fmt.Errorf("failed to run workflow: %s", err)
+		return fmt.Errorf("unable to run workflow: %w", err)
 	}
 
 	executionDetails := struct {
@@ -160,12 +158,15 @@ func StartWorkflow(c *cli.Context, printProgress bool) error {
 	}
 	fmt.Println(color.Magenta(c, "Running execution:"))
 	opts := &output.PrintOptions{
-		Fields:      []string{"WorkflowId", "RunId", "Type", "Namespace", "TaskQueue", "Args"},
-		IgnoreFlags: true,
-		Output:      output.Card,
-		Separator:   "",
+		Fields:       []string{"WorkflowId", "RunId", "Type", "Namespace", "TaskQueue", "Args"},
+		ForceFields:  true,
+		OutputFormat: output.Card,
+		Separator:    "",
+	}
+	err = output.PrintItems(c, data, opts)
+	if err != nil {
+		return err
 	}
-	output.PrintItems(c, data, opts)
 
 	if printProgress {
 		return printWorkflowProgress(c, wid, resp.GetRunID(), true)
@@ -184,83 +185,60 @@ func formatInputsForDisplay(inputs []interface{}) string {
 }
 
 func unmarshalSearchAttrFromCLI(c *cli.Context) (map[string]interface{}, error) {
-	sanitize := func(val []string) []string {
-		result := make([]string, len(val))
-		for i, v := range val {
-			result[i] = strings.TrimSpace(v)
-		}
-		return result
+	raw := c.StringSlice(FlagSearchAttribute)
+	parsed, err := SplitKeyValuePairs(raw)
+	if err != nil {
+		return nil, err
 	}
 
-	searchAttrKeys := sanitize(c.StringSlice(FlagSearchAttributeKey))
-	if len(searchAttrKeys) == 0 {
-		return nil, nil
-	}
-	rawSearchAttrVals := sanitize(c.StringSlice(FlagSearchAttributeValue))
-	if len(rawSearchAttrVals) == 0 {
-		return nil, nil
-	}
-
-	if len(searchAttrKeys) != len(rawSearchAttrVals) {
-		return nil, fmt.Errorf("uneven number of search attributes keys (%d): %v and values (%d): %v", len(searchAttrKeys), searchAttrKeys, len(rawSearchAttrVals), rawSearchAttrVals)
-	}
-
-	fields := make(map[string]interface{}, len(searchAttrKeys))
-
-	for i, v := range rawSearchAttrVals {
+	attributes := make(map[string]interface{}, len(parsed))
+	for k, v := range parsed {
 		var j interface{}
 		if err := json.Unmarshal([]byte(v), &j); err != nil {
-			return nil, fmt.Errorf("unable to parse search attribute JSON: %s", err)
+			return nil, fmt.Errorf("unable to parse Search Attribute JSON: %w", err)
 		}
-		fields[searchAttrKeys[i]] = j
+		attributes[k] = j
 	}
 
-	return fields, nil
+	return attributes, nil
 }
 
 func unmarshalMemoFromCLI(c *cli.Context) (map[string]interface{}, error) {
-	// Memo flags were not passed => Memo is not provided.
-	if !c.IsSet(FlagMemoKey) && !c.IsSet(FlagMemo) && !c.IsSet(FlagMemoFile) {
+	if !c.IsSet(FlagMemo) && !c.IsSet(FlagMemoFile) {
 		return nil, nil
 	}
 
-	if !c.IsSet(FlagMemoKey) {
-		return nil, fmt.Errorf("memo keys must be provided using %s", FlagMemoKey)
-	}
+	raw := c.StringSlice(FlagMemo)
 
-	if c.IsSet(FlagMemo) && c.IsSet(FlagMemoFile) {
-		return nil, fmt.Errorf("provide only one of %s or %s", FlagMemo, FlagMemoFile)
-	}
-
-	if !c.IsSet(FlagMemo) && !c.IsSet(FlagMemoFile) {
-		return nil, fmt.Errorf("memo values must be provided using %s or %s", FlagMemo, FlagMemoFile)
-	}
-
-	memoKeys := c.StringSlice(FlagMemoKey)
-
-	var memoValues []string
+	var rawFromFile []string
 	if c.IsSet(FlagMemoFile) {
 		inputFile := c.String(FlagMemoFile)
-		// This method is purely used to parse input from the CLI. The input comes from a trusted user
+		// The input comes from a trusted user
 		// #nosec
 		data, err := os.ReadFile(inputFile)
 		if err != nil {
 			return nil, fmt.Errorf("unable to read memo file %s", inputFile)
 		}
-		memoValues = strings.Split(string(data), "\n")
-	} else if c.IsSet(FlagMemo) {
-		memoValues = c.StringSlice(FlagMemo)
+		rawFromFile = strings.Split(string(data), "\n")
 	}
 
-	if len(memoKeys) != len(memoValues) {
-		return nil, fmt.Errorf("number of memo keys %d and values %d are not equal", len(memoKeys), len(memoValues))
+	raw = append(raw, rawFromFile...)
+
+	parsed, err := SplitKeyValuePairs(raw)
+	if err != nil {
+		return nil, err
 	}
 
-	fields := make(map[string]interface{}, len(memoKeys))
-	for i, key := range memoKeys {
-		fields[key] = memoValues[i]
+	memo := make(map[string]interface{}, len(parsed))
+	for k, v := range parsed {
+		var j interface{}
+		if err := json.Unmarshal([]byte(v), &j); err != nil {
+			return nil, fmt.Errorf("unable to parse Search Attribute JSON: %w", err)
+		}
+		memo[k] = j
 	}
-	return fields, nil
+
+	return memo, nil
 }
 
 type historyIterator struct {
@@ -294,10 +272,10 @@ func (h *historyIterator) Next() (interface{}, error) {
 
 // helper function to print workflow progress with time refresh every second
 func printWorkflowProgress(c *cli.Context, wid, rid string, watch bool) error {
-	outputFormat := output.Table
+	isJSON := false
 	if c.IsSet(output.FlagOutput) {
 		outputFlag := c.String(output.FlagOutput)
-		outputFormat = output.OutputOption(outputFlag)
+		isJSON = outputFlag == string(output.JSON)
 	}
 
 	var maxFieldLength = c.Int(FlagMaxFieldLength)
@@ -313,21 +291,22 @@ func printWorkflowProgress(c *cli.Context, wid, rid string, watch bool) error {
 	timeElapse := 1
 	isTimeElapseExist := false
 	ticker := time.NewTicker(time.Second).C
-	opts := &output.PrintOptions{
-		Fields:     []string{"ID", "Time", "Type"},
-		FieldsLong: []string{"Details"},
-	}
-	if outputFormat != output.JSON {
+	if !isJSON {
 		fmt.Println(color.Magenta(c, "Progress:"))
 	}
 
 	var lastEvent historypb.HistoryEvent // used for print result of this run
 
+	po := &output.PrintOptions{
+		Fields:     []string{"ID", "Time", "Type"},
+		FieldsLong: []string{"Details"},
+		Pager:      pager.Less,
+	}
 	errChan := make(chan error)
 	go func() {
 		hIter := sdkClient.GetWorkflowHistory(tcCtx, wid, rid, watch, enumspb.HISTORY_EVENT_FILTER_TYPE_ALL_EVENT)
 		iter := &historyIterator{iter: hIter, maxFieldLength: maxFieldLength, lastEvent: &lastEvent}
-		err = output.Pager(c, iter, opts)
+		err = output.PrintIterator(c, iter, po)
 		if err != nil {
 			errChan <- err
 			return
@@ -343,7 +322,7 @@ func printWorkflowProgress(c *cli.Context, wid, rid string, watch bool) error {
 				continue
 			}
 
-			if outputFormat != output.JSON {
+			if !isJSON {
 				if isTimeElapseExist {
 					removePrevious2LinesFromTerminal()
 				}
@@ -353,7 +332,7 @@ func printWorkflowProgress(c *cli.Context, wid, rid string, watch bool) error {
 			isTimeElapseExist = true
 			timeElapse++
 		case <-doneChan: // print result of this run
-			if outputFormat != output.JSON {
+			if !isJSON {
 				fmt.Println(color.Magenta(c, "\nResult:"))
 				if watch {
 					fmt.Printf("  Run Time: %d seconds\n", timeElapse)
@@ -367,14 +346,25 @@ func printWorkflowProgress(c *cli.Context, wid, rid string, watch bool) error {
 	}
 }
 
-// TerminateWorkflow terminates a workflow execution
 func TerminateWorkflow(c *cli.Context) error {
+	if c.String(FlagQuery) != "" {
+		return BatchTerminate(c)
+	} else {
+		return terminateWorkflow(c)
+	}
+}
+
+// TerminateWorkflow terminates a workflow execution
+func terminateWorkflow(c *cli.Context) error {
 	sdkClient, err := getSDKClient(c)
 	if err != nil {
 		return err
 	}
 
-	wid := c.String(FlagWorkflowID)
+	wid, err := requiredFlag(c, FlagWorkflowID)
+	if err != nil {
+		return err
+	}
 	rid := c.String(FlagRunID)
 	reason := c.String(FlagReason)
 
@@ -382,7 +372,7 @@ func TerminateWorkflow(c *cli.Context) error {
 	defer cancel()
 	err = sdkClient.TerminateWorkflow(ctx, wid, rid, reason, nil)
 	if err != nil {
-		return fmt.Errorf("unable to terminate workflow: %s", err)
+		return fmt.Errorf("unable to terminate workflow: %w", err)
 	}
 
 	fmt.Println("Terminate workflow succeeded")
@@ -392,17 +382,17 @@ func TerminateWorkflow(c *cli.Context) error {
 
 // DeleteWorkflow deletes a workflow execution.
 func DeleteWorkflow(c *cli.Context) error {
-	nsName, err := getRequiredGlobalOption(c, FlagNamespace)
+	nsName, err := requiredFlag(c, FlagNamespace)
 	if err != nil {
 		return err
 	}
 	wid := c.String(FlagWorkflowID)
 	rid := c.String(FlagRunID)
 
-	client := cFactory.OperatorClient(c)
+	client := cFactory.FrontendClient(c)
 	ctx, cancel := newContext(c)
 	defer cancel()
-	_, err = client.DeleteWorkflowExecution(ctx, &operatorservice.DeleteWorkflowExecutionRequest{
+	_, err = client.DeleteWorkflowExecution(ctx, &workflowservice.DeleteWorkflowExecutionRequest{
 		Namespace: nsName,
 		WorkflowExecution: &commonpb.WorkflowExecution{
 			WorkflowId: wid,
@@ -419,37 +409,59 @@ func DeleteWorkflow(c *cli.Context) error {
 	return nil
 }
 
-// CancelWorkflow cancels a workflow execution
 func CancelWorkflow(c *cli.Context) error {
+	if c.String(FlagQuery) != "" {
+		return BatchCancel(c)
+	} else {
+		return cancelWorkflow(c)
+	}
+}
+
+// cancelWorkflow cancels a workflow execution
+func cancelWorkflow(c *cli.Context) error {
 	sdkClient, err := getSDKClient(c)
 	if err != nil {
 		return err
 	}
 
-	wid := c.String(FlagWorkflowID)
+	wid, err := requiredFlag(c, FlagWorkflowID)
+	if err != nil {
+		return err
+	}
 	rid := c.String(FlagRunID)
 
 	ctx, cancel := newContext(c)
 	defer cancel()
 	err = sdkClient.CancelWorkflow(ctx, wid, rid)
 	if err != nil {
-		return fmt.Errorf("unable to cancel workflow: %s", err)
+		return fmt.Errorf("unable to cancel workflow: %w", err)
 	}
 	fmt.Println(color.Green(c, "canceled workflow, workflow id: %s, run id: %s", wid, rid))
 
 	return nil
 }
 
-// SignalWorkflow signals a workflow execution
 func SignalWorkflow(c *cli.Context) error {
+	if c.String(FlagQuery) != "" {
+		return BatchSignal(c)
+	} else {
+		return signalWorkflow(c)
+	}
+}
+
+// signalWorkflow signals a workflow execution
+func signalWorkflow(c *cli.Context) error {
 	serviceClient := cFactory.FrontendClient(c)
 
-	namespace, err := getRequiredGlobalOption(c, FlagNamespace)
+	namespace, err := requiredFlag(c, FlagNamespace)
 	if err != nil {
 		return err
 	}
 
-	wid := c.String(FlagWorkflowID)
+	wid, err := requiredFlag(c, FlagWorkflowID)
+	if err != nil {
+		return err
+	}
 	rid := c.String(FlagRunID)
 	name := c.String(FlagName)
 	input, err := processJSONInput(c)
@@ -471,7 +483,7 @@ func SignalWorkflow(c *cli.Context) error {
 	})
 
 	if err != nil {
-		return fmt.Errorf("signal workflow failed: %s", err)
+		return fmt.Errorf("signal workflow failed: %w", err)
 	}
 
 	fmt.Println("Signal workflow succeeded")
@@ -481,7 +493,7 @@ func SignalWorkflow(c *cli.Context) error {
 
 // QueryWorkflow query workflow execution
 func QueryWorkflow(c *cli.Context) error {
-	queryType := c.String(FlagQueryType)
+	queryType := c.String(FlagType)
 
 	if err := queryWorkflowHelper(c, queryType); err != nil {
 		return err
@@ -498,7 +510,7 @@ func QueryWorkflowUsingStackTrace(c *cli.Context) error {
 func queryWorkflowHelper(c *cli.Context, queryType string) error {
 	serviceClient := cFactory.FrontendClient(c)
 
-	namespace, err := getRequiredGlobalOption(c, FlagNamespace)
+	namespace, err := requiredFlag(c, FlagNamespace)
 	if err != nil {
 		return err
 	}
@@ -538,7 +550,7 @@ func queryWorkflowHelper(c *cli.Context, queryType string) error {
 	}
 	queryResponse, err := serviceClient.QueryWorkflow(tcCtx, queryRequest)
 	if err != nil {
-		return fmt.Errorf("query workflow failed: %s", err)
+		return fmt.Errorf("query workflow failed: %w", err)
 	}
 
 	if queryResponse.QueryRejected != nil {
@@ -555,10 +567,6 @@ func queryWorkflowHelper(c *cli.Context, queryType string) error {
 func ListWorkflow(c *cli.Context) error {
 	archived := c.Bool(FlagArchive)
 
-	namespace, err := getRequiredGlobalOption(c, FlagNamespace)
-	if err != nil {
-		return err
-	}
 	sdkClient, err := getSDKClient(c)
 	if err != nil {
 		return err
@@ -567,12 +575,12 @@ func ListWorkflow(c *cli.Context) error {
 	paginationFunc := func(npt []byte) ([]interface{}, []byte, error) {
 		var items []interface{}
 		var err error
-		query := c.String(FlagListQuery)
+		query := c.String(FlagQuery)
 
 		if archived {
-			items, npt, err = listArchivedWorkflows(c, sdkClient, npt, namespace, query)
+			items, npt, err = listArchivedWorkflows(c, sdkClient, npt, query)
 		} else {
-			items, npt, err = listWorkflows(c, sdkClient, npt, namespace, query)
+			items, npt, err = listWorkflows(c, sdkClient, npt, query)
 		}
 
 		if err != nil {
@@ -586,63 +594,9 @@ func ListWorkflow(c *cli.Context) error {
 	opts := &output.PrintOptions{
 		Fields:     []string{"Status", "Execution.WorkflowId", "Type.Name", "StartTime"},
 		FieldsLong: []string{"CloseTime", "Execution.RunId", "TaskQueue"},
+		Pager:      pager.Less,
 	}
-	return output.Pager(c, iter, opts)
-}
-
-// ScanAllWorkflow list all workflow executions using Scan API.
-func ScanAllWorkflow(c *cli.Context) error {
-	namespace, err := getRequiredGlobalOption(c, FlagNamespace)
-	if err != nil {
-		return err
-	}
-	listQuery := c.String(FlagListQuery)
-	sdkClient, err := getSDKClient(c)
-	if err != nil {
-		return err
-	}
-
-	paginationFunc := func(npt []byte) ([]interface{}, []byte, error) {
-		req := &workflowservice.ScanWorkflowExecutionsRequest{
-			Namespace:     namespace,
-			NextPageToken: npt,
-			Query:         listQuery,
-		}
-
-		var workflows *workflowservice.ScanWorkflowExecutionsResponse
-		op := func() error {
-			ctx, cancel := newContext(c)
-			defer cancel()
-			response, err := sdkClient.ScanWorkflow(ctx, req)
-			if err != nil {
-				return err
-			}
-			workflows = response
-			return nil
-		}
-		err := backoff.Retry(op, common.CreateFrontendServiceRetryPolicy(), common.IsContextDeadlineExceededErr)
-		if err != nil {
-			return nil, nil, fmt.Errorf("unable to list workflow executions: %s", err)
-		}
-
-		var items []interface{}
-		for _, e := range workflows.Executions {
-			items = append(items, e)
-		}
-		if err != nil {
-			return nil, nil, err
-		}
-
-		return items, workflows.NextPageToken, nil
-	}
-
-	iter := collection.NewPagingIterator(paginationFunc)
-	opts := &output.PrintOptions{
-		Fields:     []string{"Execution.WorkflowId", "Execution.RunId", "StartTime"},
-		FieldsLong: []string{"Type.Name", "TaskQueue", "ExecutionTime", "CloseTime"},
-	}
-
-	return output.Pager(c, iter, opts)
+	return output.PrintIterator(c, iter, opts)
 }
 
 // CountWorkflow count number of workflows
@@ -652,7 +606,7 @@ func CountWorkflow(c *cli.Context) error {
 		return err
 	}
 
-	query := c.String(FlagListQuery)
+	query := c.String(FlagQuery)
 	request := &workflowservice.CountWorkflowExecutionsRequest{
 		Query: query,
 	}
@@ -668,9 +622,9 @@ func CountWorkflow(c *cli.Context) error {
 		count = response.GetCount()
 		return nil
 	}
-	err = backoff.Retry(op, common.CreateFrontendServiceRetryPolicy(), common.IsContextDeadlineExceededErr)
+	err = backoff.ThrottleRetry(op, common.CreateFrontendClientRetryPolicy(), common.IsContextDeadlineExceededErr)
 	if err != nil {
-		return fmt.Errorf("unable to count workflows: %s", err)
+		return fmt.Errorf("unable to count workflows: %w", err)
 	}
 	fmt.Println(count)
 	return nil
@@ -682,7 +636,7 @@ func DescribeWorkflow(c *cli.Context) error {
 	rid := c.String(FlagRunID)
 
 	frontendClient := cFactory.FrontendClient(c)
-	namespace, err := getRequiredGlobalOption(c, FlagNamespace)
+	namespace, err := requiredFlag(c, FlagNamespace)
 	if err != nil {
 		return err
 	}
@@ -701,7 +655,7 @@ func DescribeWorkflow(c *cli.Context) error {
 		},
 	})
 	if err != nil {
-		return fmt.Errorf("workflow describe failed: %s", err)
+		return fmt.Errorf("workflow describe failed: %w", err)
 	}
 
 	if printResetPointsOnly {
@@ -845,32 +799,6 @@ func printRunStatus(c *cli.Context, event *historypb.HistoryEvent) {
 	}
 }
 
-func scanWorkflowExecutions(sdkClient sdkclient.Client, pageSize int, nextPageToken []byte, query string, c *cli.Context) ([]*workflowpb.WorkflowExecutionInfo, []byte, error) {
-	request := &workflowservice.ScanWorkflowExecutionsRequest{
-		PageSize:      int32(pageSize),
-		NextPageToken: nextPageToken,
-		Query:         query,
-	}
-
-	var workflows *workflowservice.ScanWorkflowExecutionsResponse
-	op := func() error {
-		ctx, cancel := newContext(c)
-		defer cancel()
-		response, err := sdkClient.ScanWorkflow(ctx, request)
-		if err != nil {
-			return err
-		}
-		workflows = response
-		return nil
-	}
-	err := backoff.Retry(op, common.CreateFrontendServiceRetryPolicy(), common.IsContextDeadlineExceededErr)
-	if err != nil {
-		return nil, nil, fmt.Errorf("failed to list workflow: %s", err)
-	}
-
-	return workflows.Executions, workflows.NextPageToken, nil
-}
-
 // ShowHistory shows the history of given workflow execution based on workflowID and runID.
 func ShowHistory(c *cli.Context) error {
 	wid := c.String(FlagWorkflowID)
@@ -883,7 +811,7 @@ func ShowHistory(c *cli.Context) error {
 
 // ResetWorkflow reset workflow
 func ResetWorkflow(c *cli.Context) error {
-	namespace, err := getRequiredGlobalOption(c, FlagNamespace)
+	namespace, err := requiredFlag(c, FlagNamespace)
 	if err != nil {
 		return err
 	}
@@ -894,7 +822,7 @@ func ResetWorkflow(c *cli.Context) error {
 	}
 	rid := c.String(FlagRunID)
 	eventID := c.Int64(FlagEventID)
-	resetType := c.String(FlagResetType)
+	resetType := c.String(FlagType)
 	extraForResetType, ok := resetTypesMap[resetType]
 	if !ok && eventID <= 0 {
 		return fmt.Errorf("specify either valid event id or reset type (one of %s)", strings.Join(mapKeysToArray(resetTypesMap), ", "))
@@ -920,7 +848,7 @@ func ResetWorkflow(c *cli.Context) error {
 	if resetType != "" {
 		resetBaseRunID, workflowTaskFinishID, err = getResetEventIDByType(ctx, c, resetType, namespace, wid, rid, frontendClient)
 		if err != nil {
-			return fmt.Errorf("getting reset event ID by type failed: %s", err)
+			return fmt.Errorf("getting reset event ID by type failed: %w", err)
 		}
 	}
 	resp, err := frontendClient.ResetWorkflowExecution(ctx, &workflowservice.ResetWorkflowExecutionRequest{
@@ -935,7 +863,7 @@ func ResetWorkflow(c *cli.Context) error {
 		ResetReapplyType:          resetReapplyTypesMap[resetReapplyType].(enumspb.ResetReapplyType),
 	})
 	if err != nil {
-		return fmt.Errorf("reset failed: %s", err)
+		return fmt.Errorf("reset failed: %w", err)
 	}
 	prettyPrintJSONObject(resp)
 	return nil
@@ -982,21 +910,21 @@ type batchResetParamsType struct {
 
 // ResetInBatch resets workflow in batch
 func ResetInBatch(c *cli.Context) error {
-	namespace, err := getRequiredGlobalOption(c, FlagNamespace)
+	namespace, err := requiredFlag(c, FlagNamespace)
 	if err != nil {
 		return err
 	}
-	resetType := c.String(FlagResetType)
+	resetType := c.String(FlagType)
 
 	inFileName := c.String(FlagInputFile)
-	query := c.String(FlagListQuery)
+	query := c.String(FlagQuery)
 	excFileName := c.String(FlagExcludeFile)
 	separator := c.String(FlagInputSeparator)
 	parallel := c.Int(FlagParallelism)
 
 	extraForResetType, ok := resetTypesMap[resetType]
 	if !ok {
-		return fmt.Errorf("reset type is not supported: %s", extraForResetType)
+		return fmt.Errorf("reset type is not supported: %v", extraForResetType)
 	} else if len(extraForResetType.(string)) > 0 {
 		value := c.String(extraForResetType.(string))
 		if len(value) == 0 {
@@ -1033,7 +961,7 @@ func ResetInBatch(c *cli.Context) error {
 		// #nosec
 		excFile, err := os.Open(excFileName)
 		if err != nil {
-			return fmt.Errorf("unable to read exclude rules: %s", err)
+			return fmt.Errorf("unable to read exclude rules: %w", err)
 		}
 		defer excFile.Close()
 		scanner := bufio.NewScanner(excFile)
@@ -1059,7 +987,7 @@ func ResetInBatch(c *cli.Context) error {
 	if len(inFileName) > 0 {
 		inFile, err := os.Open(inFileName)
 		if err != nil {
-			return fmt.Errorf("unable to open input file: %s", err)
+			return fmt.Errorf("unable to open input file: %w", err)
 		}
 		defer inFile.Close()
 		scanner := bufio.NewScanner(inFile)
@@ -1099,18 +1027,23 @@ func ResetInBatch(c *cli.Context) error {
 			return err
 		}
 
-		pageSize := 1000
 		var nextPageToken []byte
-		var result []*workflowpb.WorkflowExecutionInfo
+		var result []any
 		for {
-			result, nextPageToken, err = scanWorkflowExecutions(sdkClient, pageSize, nextPageToken, query, c)
+			result, nextPageToken, err = listWorkflows(c, sdkClient, nextPageToken, query)
 			if err != nil {
 				return err
 			}
-			for _, we := range result {
+			for _, resultItem := range result {
+				we, ok := resultItem.(*workflowpb.WorkflowExecutionInfo)
+				if !ok {
+					fmt.Printf("skip by wrong type:%T instead of:%T\n", resultItem, &workflowpb.WorkflowExecutionInfo{})
+					continue
+				}
+
 				wid := we.Execution.GetWorkflowId()
 				rid := we.Execution.GetRunId()
-				_, ok := excludes[wid]
+				_, ok = excludes[wid]
 				if ok {
 					fmt.Println("skip by exclude file: ", wid, rid)
 					continue
@@ -1278,12 +1211,6 @@ func getResetEventIDByType(ctx context.Context, c *cli.Context, resetType, names
 		if err != nil {
 			return
 		}
-	case "BadBinary":
-		binCheckSum := c.String(FlagResetBadBinaryChecksum)
-		resetBaseRunID, workflowTaskFinishID, err = getBadWorkflowTaskCompletedID(ctx, namespace, wid, rid, binCheckSum, frontendClient)
-		if err != nil {
-			return
-		}
 	default:
 		panic("not supported resetType")
 	}
@@ -1327,39 +1254,6 @@ func getLastWorkflowTaskEventID(ctx context.Context, namespace, wid, rid string,
 	return
 }
 
-func badChecksum(bad string) func(string) error {
-	return func(maybeBad string) error {
-		if maybeBad == bad {
-			return fmt.Errorf("bad checksum %q", bad)
-		}
-		return nil
-	}
-}
-
-func getBadWorkflowTaskCompletedID(ctx context.Context, namespace, wid, rid, binChecksum string, frontendClient workflowservice.WorkflowServiceClient) (resetBaseRunID string, workflowTaskCompletedID int64, err error) {
-	resetBaseRunID = rid
-	resp, err := frontendClient.DescribeWorkflowExecution(ctx, &workflowservice.DescribeWorkflowExecutionRequest{
-		Namespace: namespace,
-		Execution: &commonpb.WorkflowExecution{
-			WorkflowId: wid,
-			RunId:      rid,
-		},
-	})
-	if err != nil {
-		return "", 0, printErrorAndReturn("DescribeWorkflowExecution failed", err)
-	}
-
-	_, p := workflow.FindAutoResetPoint(clock.NewRealTimeSource(), badChecksum(binChecksum), resp.WorkflowExecutionInfo.AutoResetPoints)
-	if p != nil {
-		workflowTaskCompletedID = p.GetFirstWorkflowTaskCompletedId()
-	}
-
-	if workflowTaskCompletedID == 0 {
-		return "", 0, printErrorAndReturn("Get BadWorkflowTaskCompletedID failed", serviceerror.NewInvalidArgument("no WorkflowTaskCompletedID"))
-	}
-	return
-}
-
 // Returns id of the first workflow task completed event or if it doesn't exist then id of the event after task scheduled event.
 func getFirstWorkflowTaskEventID(ctx context.Context, namespace, wid, rid string, frontendClient workflowservice.WorkflowServiceClient) (resetBaseRunID string, workflowTaskEventID int64, err error) {
 	resetBaseRunID = rid
@@ -1452,9 +1346,8 @@ func getLastContinueAsNewID(ctx context.Context, namespace, wid, rid string, fro
 	return
 }
 
-func listWorkflows(c *cli.Context, sdkClient sdkclient.Client, npt []byte, namespace string, query string) ([]interface{}, []byte, error) {
+func listWorkflows(c *cli.Context, sdkClient sdkclient.Client, npt []byte, query string) ([]interface{}, []byte, error) {
 	req := &workflowservice.ListWorkflowExecutionsRequest{
-		Namespace:     namespace,
 		NextPageToken: npt,
 		Query:         query,
 	}
@@ -1470,9 +1363,9 @@ func listWorkflows(c *cli.Context, sdkClient sdkclient.Client, npt []byte, names
 		workflows = resp
 		return nil
 	}
-	err := backoff.Retry(op, common.CreateFrontendServiceRetryPolicy(), common.IsContextDeadlineExceededErr)
+	err := backoff.ThrottleRetry(op, common.CreateFrontendClientRetryPolicy(), common.IsContextDeadlineExceededErr)
 	if err != nil {
-		return nil, nil, fmt.Errorf("unable to list workflow executions: %s", err)
+		return nil, nil, fmt.Errorf("unable to list workflow executions: %w", err)
 	}
 
 	var items []interface{}
@@ -1483,9 +1376,8 @@ func listWorkflows(c *cli.Context, sdkClient sdkclient.Client, npt []byte, names
 	return items, workflows.NextPageToken, nil
 }
 
-func listArchivedWorkflows(c *cli.Context, sdkClient sdkclient.Client, npt []byte, namespace string, query string) ([]interface{}, []byte, error) {
+func listArchivedWorkflows(c *cli.Context, sdkClient sdkclient.Client, npt []byte, query string) ([]interface{}, []byte, error) {
 	req := &workflowservice.ListArchivedWorkflowExecutionsRequest{
-		Namespace:     namespace,
 		NextPageToken: npt,
 		Query:         query,
 	}
@@ -1507,9 +1399,9 @@ func listArchivedWorkflows(c *cli.Context, sdkClient sdkclient.Client, npt []byt
 		workflows = resp
 		return nil
 	}
-	err := backoff.Retry(op, common.CreateFrontendServiceRetryPolicy(), common.IsContextDeadlineExceededErr)
+	err := backoff.ThrottleRetry(op, common.CreateFrontendClientRetryPolicy(), common.IsContextDeadlineExceededErr)
 	if err != nil {
-		return nil, nil, fmt.Errorf("unable to list archived workflow executions: %s", err)
+		return nil, nil, fmt.Errorf("unable to list archived workflow executions: %w", err)
 	}
 
 	var items []interface{}
@@ -1520,6 +1412,15 @@ func listArchivedWorkflows(c *cli.Context, sdkClient sdkclient.Client, npt []byt
 	return items, workflows.NextPageToken, nil
 }
 
+func TraceWorkflow(c *cli.Context) error {
+	_, err := parseFoldStatusList(c.String(FlagFold))
+	if err != nil {
+		return err
+	}
+	fmt.Println("Trace hasn't been implemented yet.")
+	return nil
+}
+
 type eventRow struct {
 	ID      string
 	Time    string
